/***************************************************************************
 * File:        ./crusde_api.h
 * Author:      Ronni Grapenthin, NORVULK & HU-BERLIN
 * Created:     20.02.2007
 * Licence:     GPLv2
 *
 * #########################################################################
 *
 * CrusDe, simulation framework for crustal deformation studies
 * Copyright (C) 2007 Ronni Grapenthin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 ****************************************************************************/

#ifndef _green_api_h	
#define _green_api_h

#include "constants.h"
#include "config.h"

#define boolean	int		/* mimic boolean */
#define true	1		/* mimic boolean true */
#define false	0		/* mimic boolean false */
// 
#ifdef __cplusplus
extern "C" {
#endif

typedef enum{
	LOAD_PLUGIN = 0,
	KERNEL_PLUGIN = 1,
	GREEN_PLUGIN = 2,
	DATAOUT_PLUGIN = 3,
	POSTPROCESS_PLUGIN = 4,
	LOADHISTORY_PLUGIN = 5
}PluginCategory;

typedef enum{
	X_FIELD = 0,
	Y_FIELD = 1,
	Z_FIELD = 2,
	ADD_FIELD = 3
}FieldName;


void 		crusde_register_param(double* param, const char* param_name, PluginCategory);
							/*register parameter with CrusDe*/
void		crusde_register_output_field(int* position, FieldName);
							/*register output field with CrusDe*/
int 		crusde_get_size_x();			/*length of the region of interest*/
int 		crusde_get_size_y();			/*width of the region of interest*/
int 		crusde_get_size_t();			/*total number of time steps*/
int 		crusde_get_gridsize();			/*side length of a grid cell*/
int 		crusde_get_min_x();			/*westernmost coordinate of ROI*/
int 		crusde_get_min_y();			/*southernmost coordinate of ROI*/
int 		crusde_get_dimensions();		/*total number of output fields*/
int 		crusde_get_displacement_dimensions();	/*total number of spatial output fields (x,y,z)*/
int 		crusde_model_time();			/*current time step*/
int 		crusde_stepsize();			/*time increment with each model step*/
int 		crusde_get_x_index();			/*index of x-displacement values in result array*/
int 		crusde_get_y_index();			/*index of y-displacement values in result array*/
int 		crusde_get_z_index();			/*index of z-displacement values in result array*/

const char* 	crusde_get_observation_file();		/*filename of points to be observed (unused)*/
const char* 	crusde_get_load_file();			/*file that contains load definition*/
const char* 	crusde_get_out_file();			/*filename for result output*/

int 		crusde_get_green_at(double** res, int x, int y);
							/*green's function coefficients at x,y*/
double 		crusde_get_load_at(int x, int y, int t);/*load at x,y,t*/
double 		crusde_constrain_load_height(double h, int x, int y, int z);
							/*constrain load h at x,y,t*/

void 		crusde_set_result(double**);		/*return pointer to model results*/
double** 	crusde_get_result();			/*pointer to model results*/
void 		crusde_set_quadrant(int);		/*get / set quadrant in coordinate system ...*/
int		crusde_get_quadrant();			/*... in which green's function is calculated*/

void 		crusde_exit(int exitcode);		/*have CrusDe terminate gracefully*/

boolean		crusde_load_history_exists(); 		/*a load history was defined, returns {true|false}*/

/*functions to request pointers to the run time function of other plugins of the same category*/
green_exec_function 		crusde_request_green_plugin(char* plugin);
load_exec_function 		crusde_request_load_plugin(char* plugin);
run_function 			crusde_request_kernel_plugin(char* plugin);
run_function 			crusde_request_postprocessor_plugin(char* plugin);
loadhistory_exec_function 	crusde_request_loadhistory_plugin(char* plugin);

	
/*void	crusde_print_green_params();
void	crusde_print_load_params();
void	crusde_print_kernel_params();
*/

/*double* crusde_get_green_array();
double* crusde_get_load_array();
*/

#ifdef __cplusplus
}
#endif

#endif // _green_api_h
